/* $Header$ */

/* Global Page */
table.page {
	font-weight: normal;
	color: #2E3436;

	font-family: "bitstream vera sans","luxi sans",verdana,geneva,arial,helvetica,sans-serif;
	background-color: #FFFFFF;
	font-size: 13px;
	empty-cells: hide;
}

/* Global Page - Defaults */
/* A HREF Links */
table.page a {
	color: #204A87;
	text-decoration: none;
}

table.page a:hover {
	text-decoration: none;
}

table.page a img {
	border: 0px;
}

/* Global Page - Logo & Title */
table.page tr.head {
	text-align: center;
	color: #FFFFFF;
	background-color: #3465A4;
	font-weight: bold;
	font-size: 11px;
	height: 25px;
}

table.page tr.head img.logo {
	vertical-align: middle;
	text-align: center;

	width: 100px;
	height: 60px;
}

table.page tr.pagehead {
}

table.page tr.pagehead td.imagetop {
	width: 100%;
	vertical-align: bottom;
	text-align: right;
}

/* Global Page - Control Line */
table.page tr.control td {
	border-top: 1px solid #BABDB6;
	border-bottom: 1px solid #BABDB6;
}

/* Global Page - Control Line Menu Items */
table.page table.control {
	table-layout: fixed;
	width: 100%;
}

table.page table.control td {
	border-top: 0px;
	border-bottom: 0px;
	padding: 0px;
	padding-top: 5px;
	text-align: left;
	vertical-align: top;
	font-size: 11px;
}

table.page table.control img {
	width: 24px;
	height: 24px;
}

table.page table.control a {
	color: #2E3436;
}

table.page table.control a:hover {
	text-decoration: none;
	background-color: #EEEEEC;
	color: #CC0000;
}

table.page table.control td.spacer {
	width: 20%;
}

table.page table.control td.logo {
	text-align: right;
	width: 10%;
}

table.page table.control td.logo img.logo {
	vertical-align: middle;
	text-align: right;

	width: 100px;
	height: 60px;
}

/* Global Page - LDAP Tree */
table.page td.tree {
	border-right: 1px solid #BABDB6;
	vertical-align: top;
	background-color: #FFFFFF;
	width: 10%;
}

/* Global Page - Main Body */
table.page td.body {
	vertical-align: top;
	width: 100%;
	background-color: #FFFFFF;
}

/* Global Page - Main Body System Message */
table.page table.sysmsg {
	border-bottom: 2px solid #BABDB6;
	width: 100%;
}

table.page table.sysmsg td.head {
	font-size: small;
	text-align: left;
	font-weight: bold;
}

table.page table.sysmsg td.body {
	font-weight: normal;
}

table.page table.sysmsg td.icon {
	text-align: center;
	vertical-align: top;
}

/* Global Page - Main Body */
table.page table.body {
	font-weight: normal;
	background-color: #FFFFFF;
	width: 100%;
}

table.page table.body h3.title {
	text-align: center;
	margin: 0px;
	padding: 10px;
	color: #FFFFFF;
	background-color: #3465A4;
	border: 1px solid #EEEEEC;
	font-weight: normal;
	font-size: 150%;
}

table.page table.body h3.subtitle {
	text-align: center;
	margin: 0px;
	margin-bottom: 15px;
	font-size: 75%;
	color: #FFFFFF;
	border-bottom: 1px solid #EEEEEC;
	border-left: 1px solid #EEEEEC;
	border-right: 1px solid #EEEEEC;
	background: #3465A4;
	padding: 4px;
	font-weight: normal;
}

table.page table.body td.spacer {
	border-top: 2px solid #BABDB6;
	padding: 0px;
	font-size: 5px;
}

table.page table.body td.head {
	font-weight: bold;
}

table.page table.body td.foot {
	font-size: small;
	border-top: 1px solid #BABDB6;
	border-bottom: 1px solid #BABDB6;
}

/* Global Page Footer */
table.page tr.foot td {
	border-top: 1px solid #BABDB6;
	font-weight: bold;
	font-size: 12px;
	text-align: right;
}

/* Global Page - Other Layouts */
/* Server Select */
table.page table.server_select {
	font-weight: bold;
	font-size: 13px;
	color: #2E3436;
}

/* Individual table layouts */
/* LDAP Tree */
table.tree {
}

table.tree tr.server td.icon {
	vertical-align: top;
}

table.tree tr.server td.name {
	padding-right: 10px;
	vertical-align: top;
}

table.tree tr.server td {
	padding-top: 5px;
	font-size: 18px;
	text-align: left;
	padding-right: 0px;
	white-space: nowrap;
}

table.tree td {
	white-space: nowrap;
}

table.tree td.server_links {
	vertical-align: top;
	text-align: center;
	padding-top: 0px;
	padding-bottom: 0px;
	padding-left: 3px;
	padding-right: 3px;
}

table.tree td.server_links img {
	height: 22px;
	width: 22px;
}

table.tree td.server_links a {
	color: #2E3436;
	text-decoration: none;
	font-size: 11px;
}

table.tree td.server_links a:hover {
	text-decoration: none;
	background-color: #EEEEEC;
	color: #CC0000;
}

table.tree tr.option td.expander {
	text-align: center;
	width: 22px;
	max-width: 22px;
	min-width: 22px;
	white-space: nowrap;
}

table.tree tr.option td.icon {
	text-align: center;
	width: 22px;
	max-width: 22px;
	min-width: 22px;
	white-space: nowrap;
}

table.tree td.rdn a {
	font-size: 13px;
	color: #2E3436;
}

table.tree td.rdn a:hover {
	font-size: 13px;
	color: #CC0000;
	background-color: #EEEEEC;
}

table.tree td.rdn span.count {
	font-size: 13px;
	color: #2E3436;
}

table.tree td.links a {
	color: #204A87;
	text-align: center;
}

table.tree td.link a {
	font-size: 13px;
	color: #2E3436;
}

table.tree td.link a:hover {
	font-size: 13px;
	color: #CC0000;
	background-color: #EEEEEC;
	text-decoration: none;
}

table.tree td.rdn a:hover {
	font-size: 13px;
	color: #CC0000;
	background-color: #EEEEEC;
	text-decoration: none;
}

table.tree td.links a:hover {
	text-decoration: none;
	color: #204A87;
}

table.tree td.links a img {
	width: 22px;
	height: 22px;
}

table.tree td.blank {
	font-size: 1px;
}

table.tree td.spacer {
	width: 22px;
}

table.tree td.logged_in {
	font-size: 10px;
	white-space: nowrap;
}

table.tree td.logged_in a {
	font-size: 11px;
}

table.tree td.logged_in a:hover {
	color: #CC0000;
	background-color: #EEEEEC;
	text-decoration: none;
}

/* Tree Global Defaults */
table.tree tr td {
	padding: 0px;
}

table.tree a {
	text-decoration: none;
	color: #2E3436;
}

table.tree a:hover {
	text-decoration: underline;
	color: #204A87;
}

/* Tree */
table.tree .treemenudiv {
	display: block;
	white-space: nowrap;
	padding-top: 1px;
	padding-bottom: 1px;
}

table.tree .phplmnormal {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #000000;
	text-decoration: none;
}

table.tree a.phplmnormal:hover {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #000000;
	background-color: #fff0c0;
	text-decoration: none;
}

table.tree a.phplm:link {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #000000;
	text-decoration: none;
}

table.tree a.phplm:visited {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #000000;
	text-decoration: none;
}

table.tree a.phplm:hover {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #841212;
	background-color: #fff0c0;
	text-decoration: none;
}

table.tree a.phplm:active {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #ff0000;
	text-decoration: none;
}

table.tree a.phplmselected:link {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #dd0000;
	background-color: #ffdd76;
	text-decoration: none;
}

table.tree a.phplmselected:visited {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #dd0000;
	background-color: #ffdd76;
	text-decoration: none;
}

table.tree a.phplmselected:hover {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #841212;
	background-color: #fff0c0;
	text-decoration: none;
}

table.tree a.phplmselected:active {
	font-family: bitstream vera sans, luxi sans, verdana, geneva, arial, helvetica, sans-serif;
	font-size: 13px;
	color: #ff0000;
	text-decoration: none;
}

/* Standard Form */
table.forminput {
	background-color: #EEEEEC;
	padding: 10px;
	border: 1px solid #BABDB6;
}

table.forminput td.title {
	text-align: center;
	font-weight: bold;
}

table.forminput td.subtitle {
	text-align: center;
	font-weight: normal;
	font-size: small;
}

table.forminput tr td.heading {
	font-weight: bold;
}

table.forminput td.small {
	font-size: 80%;
}

table.forminput td.top {
	vertical-align: top;
}

table.forminput input.val {
	width: 350px;
	border: 1px solid #BABDB6;
}

table.forminput input.roval {
	width: 350px;
	border: none;
}

table.forminput td.icon {
	width: 16px;
	text-align: center;
}

table.forminput td.icon img {
	border: 0px;
}

table.forminput td.label {
	text-align: left;
	font-size: 13px;
}

/* Menu on top of entry form */
table.menu {
	font-size: 14px;
}

table.menu td.icon {
	width: 16px;
	text-align: center;
}

/* Edit DN */
div.add_value {
	font-size: 12px;
	margin: 0px;
	padding: 0px;
}

/* Edit Entry */
table.entry {
	border-collapse: collapse;
	border-spacing: 0px;
	empty-cells: show;
}

table.entry input {
	margin: 1px;
}

table.entry input.value {
	font-size: 14px;
	width: 350px;
	background-color: #FFFFFF;
}

table.entry div.helper {
	text-align: left;
	white-space: nowrap;
	background-color: #FFFFFF;
	font-size: 14px;
	font-weight: normal;
	color: #888;
}

table.entry input.roval {
	font-size: 14px;
	width: 350px;
	background-color: #FFFFFF;
	border: none;
}

table.entry textarea.value {
	font-size: 14px;
	width: 350px;
	background-color: #FFFFFF;
}

table.entry textarea.roval {
	font-size: 14px;
	width: 350px;
	background-color: #FFFFFF;
	border: none;
}

table.entry tr td {
	padding: 4px;
	padding-right: 0px;
}

table.entry tr td.heading {
	border-top: 3px solid #D3D7CF;
	font-weight: bold;
}

table.entry tr td.note {
	text-align: right;
	background-color: #EEEEEC;
}

table.entry tr td.title {
	background-color: #EEEEEC;
	vertical-align: top;
	font-weight: bold;
}

table.entry tr td.title a {
	text-decoration: none;
	color: #2E3436;
}

table.entry tr td.title a:hover {
	text-decoration: underline;
	color: #204A87;
}

table.entry tr td.value {
	text-align: left;
	vertical-align: middle;
	padding-bottom: 10px;
	padding-left: 50px;
}

/** When an attr is updated, it is highlighted to indicate such */
table.entry tr.updated td.title {
	border-top: 1px dashed #BABDB6;
	border-left: 1px dashed #BABDB6;
	background-color: #888A85;
}

table.entry tr.updated td.note {
	border-top: 1px dashed #BABDB6;
	border-right: 1px dashed #BABDB6;
	background-color: #888A85;
}

/** An extra row that sits at the bottom of recently modified attrs to encase them in dashes */
table.entry tr.updated td.bottom {
	border-top: 1px dashed #BABDB6;
}

/** Formatting for the value cell when it is the attribute that has been recently modified */
table.entry tr.updated td.value {
	border-left: 1px dashed #BABDB6;
	border-right: 1px dashed #BABDB6;
}

/* Need to prevent sub-tables (like the one in which jpegPhotos are displayed)
 * from drawing borders as well. */
table.entry tr.updated td table td {
	border: 0px;
}

table.entry tr.noinput {
	background: #EEEEEC;
}

span.hint {
	font-size: small;
	font-weight: normal;
	color: #888;
}

/* Edit DN - EntryWriter2 */
table.entry tr.spacer {
	background-color: #D3D7CF;
}

table.entry tr td.ew2_icon {
	vertical-align: top;
}

table.entry tr td.ew2_attr {
	vertical-align: top;
	text-align: right;
	font-size: 75%;
	background-color: #FFFFFF;
	font-weight: bold;
}

table.entry tr td.ew2_attr a {
	text-decoration: none;
	color: #2E3436;
}

table.entry tr td.ew2_attr a:hover {
	text-decoration: underline;
	color: #204A87;
}

table.entry tr td.ew2_val {
	text-align: left;
	vertical-align: top;
	padding-bottom: 10px;
	padding-left: 50px;
}

table.entry tr.updated td.ew2_attr {
	text-align: right;
	font-size: 75%;
	border-top: 1px dashed green;
	border-left: 1px dashed green;
	border-bottom: 1px dashed green;
	background-color: #ded;
}

table.entry tr.updated td.ew2_val {
	border-top: 1px dashed green;
	border-left: 1px dashed green;
	border-right: 1px dashed green;
	border-bottom: 1px dashed green;
}

/* Login Box */
#login {
	background: url('../../images/tango/ldap-uid.png') no-repeat 0 1px;
	background-color: #FFFFFF;
	padding-left: 17px;
}

#login:focus {
	background-color: #EEEEEC;
}

#login:disabled {
	background-color: #D3D7CF;
}

#password {
	background: url('../../images/tango/key.png') no-repeat 0 1px;
	background-color: #FFFFFF;
	padding-left: 17px;
}

#password:focus {
	background-color: #EEEEEC;
}

#password:disabled {
	background-color: #D3D7CF;
}

#generic {
	background-color: #FFFFFF;
	padding-left: 17px;
}
#generic:focus {
	background-color: #EEEEEC;
}

#generic:disabled {
	background-color: #D3D7CF;
}

/* After input results */
div.execution_time {
	font-size: 75%;
	font-weight: normal;
	text-align: center;
}

table.result {
	width: 100%;
	vertical-align: top;
	empty-cells: show;
	border: 1px solid #BABDB6;
	border-spacing: 0px;
	background-color: #EEEEEC;
}

table.result tr.heading {
	vertical-align: top;
}

table.result tr.list_title {
	background-color: #FFFFFF;
}

table.result tr.list_title td.icon {
	text-align: center;
	vertical-align: top;
}

table.result tr.list_item {
	background-color: #FFFFFF;
}

table.result tr.list_item td.blank {
	width: 25px;
}

table.result tr.list_item td.heading {
	vertical-align: top;
	color: gray;
	width: 10%;
	font-size: 12px;
}

table.result tr.list_item td.value {
	color: #2E3436;
	font-size: 12px;
}

table.result_table {
	border: 1px solid #BABDB6;
	border-collapse: collapse;
	empty-cells: show;
}

table.result_table td {
	vertical-align: top;
	border: 1px solid #BABDB6;
	padding: 4px;
}

table.result_table th {
	border: 1px solid #BABDB6;
	padding: 10px;
	padding-left: 20px;
	padding-right: 20px;
}

table.result_table tr.highlight {
	background-color: #FCE94F;
}

table.result_table tr.highlight td {
	border: 1px solid #BABDB6;
	font-weight: bold;
	padding-top: 5px;
	padding-bottom: 5px;
	padding-left: 10px;
	padding-right: 10px;
}

table.result_table td.heading {
	color: #FFFFFF;
	background-color: #3465A4;
	font-size: 15px;
}

table.result_table td.value {
	color: #2E3436;
	background-color: #EEEEEC;
}

table.result_table tr.heading {
	color: #FFFFFF;
	background-color: #3465A4;
	font-size: 15px;
}

table.result_table tr.heading a {
	color: #FFFFFF;
	font-size: 20px;
}

table.result_table tr.heading td {
	border: 1px solid #BABDB6;
	font-weight: normal;
	padding-top: 5px;
	padding-bottom: 5px;
	padding-left: 10px;
	padding-right: 10px;
}

table.result_table tr.even {
	background-color: #EEEEEC;
}

table.result_table tr.even td {
	border: 1px solid #BABDB6;
	font-weight: normal;
	padding-top: 5px;
	padding-bottom: 5px;
	padding-left: 10px;
	padding-right: 10px;
}

table.result_table tr.even td.title {
	font-weight: bold;
}

table.result_table tr.odd {
	background-color: #EEEEEC;
}

table.result_table tr.odd td {
	border: 1px solid #BABDB6;
	font-weight: normal;
	padding-top: 5px;
	padding-bottom: 5px;
	padding-left: 10px;
	padding-right: 10px;
}

table.result_table tr.odd td.title {
	font-weight: bold;
}

table.result_table ul.list {
	margin: 5px;
	margin-left: 0px;
	padding-left: 20px;
}

table.result_table ul.list li {
	margin-left: 0px;
	padding-left: 0px;
}

table.result_table ul.list li small {
	font-size: 75%;
	color: #707070;
}

table.result_table ul.list li small a {
	color: #7070C0;
}

/* Error Dialog Box */
table.error {
	width: 500px;
	border: 1px solid #AA0000;
	background-color: #FFF0F0;
}

table.error th {
	background-color: #AA0000;
	border: 0px;
	color: #FFFFFF;
	font-size: 14px;
	font-weight: bold;
	text-align: center;
	vertical-align: middle;
	width: 100%;
}

table.error th.img {
	vertical-align: middle;
	text-align: center;
}

table.error td {
	border: 0px;
	background-color: #FFF0F0;
	padding: 2px;
	text-align: left;
	vertical-align: top;
}

/* Popup Window */
div.popup h3.subtitle {
	text-align: center;
	margin: 0px;
	margin-bottom: 15px;
	color: #FFFFFF;
	border-bottom: 1px solid #2E3436;
	border-left: 1px solid #2E3436;
	border-right: 1px solid #2E3436;
	background: #3465A4;
	padding: 4px;
	font-weight: normal;
}

span.good {
	color: green;
}

span.bad {
	color: red;
}
