<?php
/**
 * Classes and functions for searching the LDAP server
 *
 * These classes and functions will read our XML search files, for defined
 * search reports.
 *
 * @author The phpLDAPadmin development team
 * @package phpLDAPadmin
 */

/**
 * Query Class
 *
 * This class will store all our pre-defined search definitions.
 *
 * @package phpLDAPadmin
 * @subpackage Queries
 */
class Queries extends xmlTemplates {
}
?>
