<?php
/**
 * Classes and functions for the template engine.
 *
 * @author The phpLDAPadmin development team
 * @package phpLDAPadmin
 */

/**
 * Represents an attribute whose values are SAMBA passwords
 *
 * @package phpLDAPadmin
 * @subpackage Templates
 */
class SambaPasswordAttribute extends PasswordAttribute {
}
?>
