<?php
/**
 * Classes and functions for the template engine.
 *
 * These classes and functions will read our XML template files, as well as
 * query the LDAP server and build a template object that can be used for user
 * input.
 *
 * @author The phpLDAPadmin development team
 * @package phpLDAPadmin
 */

/**
 * Templates Class
 *
 * This class will store all our templates.
 *
 * @package phpLDAPadmin
 * @subpackage Templates
 */
class Templates extends xmlTemplates {
}
?>
